@@ -1,14 +1,9 @@
-from __future__ import division, absolute_import, print_function
-
 import numpy as np
+from .common import Benchmark, safe_import
 
-try:
+with safe_import():
     import scipy.linalg.lapack as la
     import scipy.linalg.blas as bla
-except ImportError:
-    pass
-
-from .common import Benchmark
 
 
 class GetBlasLapackFuncs(Benchmark):
