@@ -1,13 +1,18 @@
 import warnings
 
 import numpy as np
+from .common import Benchmark, safe_import
 
-try:
+with safe_import():
     import scipy.stats as stats
+with safe_import():
+    from scipy.stats._distr_params import distcont
+
+try:  # builtin lib
+    from itertools import compress
 except ImportError:
     pass
 
-from .common import Benchmark
 
 class Anderson_KSamp(Benchmark):
     def setup(self, *args):
@@ -152,23 +157,74 @@ class GroupSampling(Benchmark):
         stats.special_ortho_group.rvs(dim)
 
 
-class BinnedStatistic(Benchmark):
+class BinnedStatisticDD(Benchmark):
 
-    def setup(self):
+    params = ["count", "sum", "mean", "min", "max", "median", "std", np.std]
+
+    def setup(self, statistic):
         np.random.seed(12345678)
         self.inp = np.random.rand(9999).reshape(3, 3333) * 200
         self.subbin_x_edges = np.arange(0, 200, dtype=np.float32)
         self.subbin_y_edges = np.arange(0, 200, dtype=np.float64)
         self.ret = stats.binned_statistic_dd(
-            [self.inp[0], self.inp[1]], self.inp[2], statistic="std",
+            [self.inp[0], self.inp[1]], self.inp[2], statistic=statistic,
             bins=[self.subbin_x_edges, self.subbin_y_edges])
 
-    def time_binned_statistic_dd_std(self):
+    def time_binned_statistic_dd(self, statistic):
         stats.binned_statistic_dd(
-            [self.inp[0], self.inp[1]], self.inp[2], statistic="std",
+            [self.inp[0], self.inp[1]], self.inp[2], statistic=statistic,
             bins=[self.subbin_x_edges, self.subbin_y_edges])
 
-    def time_binned_statistic_dd_std_reuse_bin(self):
+    def time_binned_statistic_dd_reuse_bin(self, statistic):
         stats.binned_statistic_dd(
-            [self.inp[0], self.inp[1]], self.inp[2], statistic="std",
+            [self.inp[0], self.inp[1]], self.inp[2], statistic=statistic,
             binned_statistic_result=self.ret)
+
+
+class ContinuousFitAnalyticalMLEOverride(Benchmark):
+    # list of distributions to time
+    dists = ["pareto", "laplace", "rayleigh",
+             "invgauss", "gumbel_r", "gumbel_l"]
+    # add custom values for rvs and fit, if desired, for any distribution:
+    # key should match name in dists and value should be list of loc, scale,
+    # and shapes
+    custom_input = {}
+    fnames = ['floc', 'fscale', 'f0', 'f1', 'f2']
+    fixed = {}
+    distcont = dict(distcont)
+
+    param_names = ["distribution", "loc_fixed", "scale_fixed",
+                   "shape1_fixed", "shape2_fixed", "shape3_fixed"]
+    params = [dists, * [[True, False]] * 5]
+
+    def setup(self, dist_name, loc_fixed, scale_fixed, shape1_fixed,
+              shape2_fixed, shape3_fixed):
+        self.distn = eval("stats." + dist_name)
+
+        # default `loc` and `scale` are .834 and 4.342, and shapes are from
+        # `_distr_params.py`
+        default_shapes = self.distcont[dist_name]
+        param_values = self.custom_input.get(dist_name, [.834, 4.342,
+                                                         *default_shapes])
+        # separate relevant and non-relevant parameters for this distribution
+        # based on the number of shapes
+        nparam = len(param_values)
+        all_parameters = [loc_fixed, scale_fixed, shape1_fixed, shape2_fixed,
+                          shape3_fixed]
+        relevant_parameters = all_parameters[:nparam]
+        nonrelevant_parameters = all_parameters[nparam:]
+
+        # skip if all parameters are fixed or if non relevant parameters are
+        # not all false
+        if True in nonrelevant_parameters or False not in relevant_parameters:
+            raise NotImplementedError("skip non-relevant case")
+
+        # add fixed values if fixed in relevant_parameters to self.fixed
+        # with keys from self.fnames and values from parameter_values
+        self.fixed = dict(zip(compress(self.fnames, relevant_parameters),
+                          compress(param_values, relevant_parameters)))
+        self.data = self.distn.rvs(*param_values, size=1000)
+
+    def time_fit(self, dist_name, loc_fixed, scale_fixed, shape1_fixed,
+                 shape2_fixed, shape3_fixed):
+        self.distn.fit(self.data, **self.fixed)
