@@ -1,13 +1,14 @@
 # -*- encoding:utf-8 -*-
-from __future__ import division, absolute_import, print_function
-
+from collections import namedtuple
+from copy import deepcopy
 import re
-import sys
 import textwrap
 import warnings
 
 import jinja2
 
+from numpydoc.numpydoc import update_config
+from numpydoc.xref import DEFAULT_LINKS
 from numpydoc.docscrape import (
     NumpyDocString,
     FunctionDoc,
@@ -16,15 +17,8 @@ from numpydoc.docscrape import (
 )
 from numpydoc.docscrape_sphinx import (SphinxDocString, SphinxClassDoc,
                                        SphinxFunctionDoc, get_doc_object)
-from nose.tools import (assert_equal, assert_raises, assert_list_equal,
-                        assert_true)
-
-assert_list_equal.__self__.maxDiff = None
-
-if sys.version_info[0] >= 3:
-    sixu = lambda s: s
-else:
-    sixu = lambda s: unicode(s, 'unicode_escape')
+from pytest import raises as assert_raises
+from pytest import warns as assert_warns
 
 
 doc_txt = '''\
@@ -122,13 +116,13 @@ doc_txt = '''\
   >>> mean = (1,2)
   >>> cov = [[1,0],[1,0]]
   >>> x = multivariate_normal(mean,cov,(3,3))
-  >>> print x.shape
+  >>> print(x.shape)
   (3, 3, 2)
 
   The following is probably true, given that 0.6 is roughly twice the
   standard deviation:
 
-  >>> print list( (x[0,0,:] - mean) < 0.6 )
+  >>> print(list((x[0, 0, :] - mean) < 0.6))
   [True, True]
 
   .. index:: random
@@ -152,6 +146,25 @@ int
 doc_yields = NumpyDocString(doc_yields_txt)
 
 
+doc_sent_txt = """
+Test generator
+
+Yields
+------
+a : int
+    The number of apples.
+
+Receives
+--------
+b : int
+    The number of bananas.
+c : int
+    The number of oranges.
+
+"""
+doc_sent = NumpyDocString(doc_sent_txt)
+
+
 def test_signature():
     assert doc['Signature'].startswith('numpy.multivariate_normal(')
     assert doc['Signature'].endswith('spam=None)')
@@ -167,52 +180,65 @@ def test_extended_summary():
 
 
 def test_parameters():
-    assert_equal(len(doc['Parameters']), 3)
-    assert_equal([n for n,_,_ in doc['Parameters']], ['mean','cov','shape'])
+    assert len(doc['Parameters']) == 3
+    names = [n for n, _, _ in doc['Parameters']]
+    assert all(a == b for a, b in zip(names, ['mean', 'cov', 'shape']))
 
     arg, arg_type, desc = doc['Parameters'][1]
-    assert_equal(arg_type, '(N, N) ndarray')
+    assert arg_type == '(N, N) ndarray'
     assert desc[0].startswith('Covariance matrix')
     assert doc['Parameters'][0][-1][-1] == '   (1+2+3)/3'
 
 
 def test_other_parameters():
-    assert_equal(len(doc['Other Parameters']), 1)
-    assert_equal([n for n,_,_ in doc['Other Parameters']], ['spam'])
+    assert len(doc['Other Parameters']) == 1
+    assert [n for n, _, _ in doc['Other Parameters']] == ['spam']
     arg, arg_type, desc = doc['Other Parameters'][0]
-    assert_equal(arg_type, 'parrot')
+    assert arg_type == 'parrot'
     assert desc[0].startswith('A parrot off its mortal coil')
 
 
 def test_returns():
-    assert_equal(len(doc['Returns']), 3)
+    assert len(doc['Returns']) == 3
     arg, arg_type, desc = doc['Returns'][0]
-    assert_equal(arg, 'out')
-    assert_equal(arg_type, 'ndarray')
+    assert arg == 'out'
+    assert arg_type == 'ndarray'
     assert desc[0].startswith('The drawn samples')
     assert desc[-1].endswith('distribution.')
 
     arg, arg_type, desc = doc['Returns'][1]
-    assert_equal(arg, 'list of str')
-    assert_equal(arg_type, '')
+    assert arg == ''
+    assert arg_type == 'list of str'
     assert desc[0].startswith('This is not a real')
     assert desc[-1].endswith('anonymous return values.')
 
     arg, arg_type, desc = doc['Returns'][2]
-    assert_equal(arg, 'no_description')
-    assert_equal(arg_type, '')
+    assert arg == ''
+    assert arg_type == 'no_description'
     assert not ''.join(desc).strip()
 
 
 def test_yields():
     section = doc_yields['Yields']
-    assert_equal(len(section), 3)
+    assert len(section) == 3
     truth = [('a', 'int', 'apples.'),
              ('b', 'int', 'bananas.'),
-             ('int', '', 'unknowns.')]
+             ('', 'int', 'unknowns.')]
     for (arg, arg_type, desc), (arg_, arg_type_, end) in zip(section, truth):
-        assert_equal(arg, arg_)
-        assert_equal(arg_type, arg_type_)
+        assert arg == arg_
+        assert arg_type == arg_type_
+        assert desc[0].startswith('The number of')
+        assert desc[0].endswith(end)
+
+
+def test_sent():
+    section = doc_sent['Receives']
+    assert len(section) == 2
+    truth = [('b', 'int', 'bananas.'),
+             ('c', 'int', 'oranges.')]
+    for (arg, arg_type, desc), (arg_, arg_type_, end) in zip(section, truth):
+        assert arg == arg_
+        assert arg_type == arg_type_
         assert desc[0].startswith('The number of')
         assert desc[0].endswith(end)
 
@@ -290,21 +316,21 @@ That should break...
         SphinxClassDoc(Dummy)
     except ValueError as e:
         # python 3 version or python 2 version
-        assert_true("test_section_twice.<locals>.Dummy" in str(e)
-                    or 'test_docscrape.Dummy' in str(e))
+        assert ("test_section_twice.<locals>.Dummy" in str(e)
+                or 'test_docscrape.Dummy' in str(e))
 
     try:
         SphinxFunctionDoc(dummy_func)
     except ValueError as e:
         # python 3 version or python 2 version
-        assert_true("test_section_twice.<locals>.dummy_func" in str(e)
-                    or 'function dummy_func' in str(e))
+        assert ("test_section_twice.<locals>.dummy_func" in str(e)
+                or 'function dummy_func' in str(e))
 
 
 def test_notes():
     assert doc['Notes'][0].startswith('Instead')
     assert doc['Notes'][-1].endswith('definite.')
-    assert_equal(len(doc['Notes']), 17)
+    assert len(doc['Notes']) == 17
 
 
 def test_references():
@@ -318,9 +344,9 @@ def test_examples():
 
 
 def test_index():
-    assert_equal(doc['index']['default'], 'random')
-    assert_equal(len(doc['index']), 2)
-    assert_equal(len(doc['index']['refguide']), 2)
+    assert doc['index']['default'] == 'random'
+    assert len(doc['index']) == 2
+    assert len(doc['index']['refguide']) == 2
 
 
 def _strip_blank_lines(s):
@@ -331,12 +357,14 @@ def _strip_blank_lines(s):
     return s
 
 
-def line_by_line_compare(a, b):
+def line_by_line_compare(a, b, n_lines=None):
     a = textwrap.dedent(a)
     b = textwrap.dedent(b)
-    a = [l.rstrip() for l in _strip_blank_lines(a).split('\n')]
-    b = [l.rstrip() for l in _strip_blank_lines(b).split('\n')]
-    assert_list_equal(a, b)
+    a = [l.rstrip() for l in _strip_blank_lines(a).split('\n')][:n_lines]
+    b = [l.rstrip() for l in _strip_blank_lines(b).split('\n')][:n_lines]
+    assert len(a) == len(b)
+    for ii, (aa, bb) in enumerate(zip(a, b)):
+        assert aa == bb
 
 
 def test_str():
@@ -404,7 +432,7 @@ See Also
 --------
 
 `some`_, `other`_, `funcs`_
-
+    ..
 `otherfunc`_
     relationship
 
@@ -440,13 +468,13 @@ Examples
 >>> mean = (1,2)
 >>> cov = [[1,0],[1,0]]
 >>> x = multivariate_normal(mean,cov,(3,3))
->>> print x.shape
+>>> print(x.shape)
 (3, 3, 2)
 
 The following is probably true, given that 0.6 is roughly twice the
 standard deviation:
 
->>> print list( (x[0,0,:] - mean) < 0.6 )
+>>> print(list((x[0, 0, :] - mean) < 0.6))
 [True, True]
 
 .. index:: random
@@ -465,9 +493,42 @@ b : int
     The number of bananas.
 int
     The number of unknowns.
+""")
 
-.. index:: """)
 
+def test_receives_str():
+    line_by_line_compare(str(doc_sent),
+"""Test generator
+
+Yields
+------
+a : int
+    The number of apples.
+
+Receives
+--------
+b : int
+    The number of bananas.
+c : int
+    The number of oranges.
+""")
+
+
+def test_no_index_in_str():
+    assert "index" not in str(NumpyDocString("""Test idx
+
+    """))
+
+    assert "index" in str(NumpyDocString("""Test idx
+
+    .. index :: random
+    """))
+
+    assert "index" in str(NumpyDocString("""Test idx
+
+    .. index ::
+        foo
+    """))
 
 def test_sphinx_str():
     sphinx_doc = SphinxDocString(doc_txt)
@@ -509,11 +570,11 @@ of the one-dimensional normal distribution to higher dimensions.
         In other words, each entry ``out[i,j,...,:]`` is an N-dimensional
         value drawn from the distribution.
 
-    **list of str**
+    list of str
         This is not a real return value.  It exists to test
         anonymous return values.
 
-    **no_description**
+    no_description
         ..
 
 :Other Parameters:
@@ -523,12 +584,12 @@ of the one-dimensional normal distribution to higher dimensions.
 
 :Raises:
 
-    **RuntimeError**
+    RuntimeError
         Some error
 
 :Warns:
 
-    **RuntimeWarning**
+    RuntimeWarning
         Some warning
 
 .. warning::
@@ -538,7 +599,7 @@ of the one-dimensional normal distribution to higher dimensions.
 .. seealso::
 
     :obj:`some`, :obj:`other`, :obj:`funcs`
-
+        ..
     :obj:`otherfunc`
         relationship
 
@@ -578,13 +639,13 @@ definite.
 >>> mean = (1,2)
 >>> cov = [[1,0],[1,0]]
 >>> x = multivariate_normal(mean,cov,(3,3))
->>> print x.shape
+>>> print(x.shape)
 (3, 3, 2)
 
 The following is probably true, given that 0.6 is roughly twice the
 standard deviation:
 
->>> print list( (x[0,0,:] - mean) < 0.6 )
+>>> print(list((x[0, 0, :] - mean) < 0.6))
 [True, True]
 """)
 
@@ -602,7 +663,7 @@ def test_sphinx_yields_str():
     **b** : int
         The number of bananas.
 
-    **int**
+    int
         The number of unknowns.
 """)
 
@@ -620,7 +681,7 @@ doc2 = NumpyDocString("""
 
 
 def test_parameters_without_extended_description():
-    assert_equal(len(doc2['Parameters']), 2)
+    assert len(doc2['Parameters']) == 2
 
 
 doc3 = NumpyDocString("""
@@ -632,13 +693,13 @@ doc3 = NumpyDocString("""
 
 def test_escape_stars():
     signature = str(doc3).split('\n')[0]
-    assert_equal(signature, 'my_signature(\*params, \*\*kwds)')
+    assert signature == r'my_signature(\*params, \*\*kwds)'
 
     def my_func(a, b, **kwargs):
         pass
 
     fdoc = FunctionDoc(func=my_func)
-    assert_equal(fdoc['Signature'], 'my_func(a, b, \*\*kwargs)')
+    assert fdoc['Signature'] == 'my_func(a, b, **kwargs)'
 
 
 doc4 = NumpyDocString(
@@ -648,7 +709,7 @@ doc4 = NumpyDocString(
 
 
 def test_empty_extended_summary():
-    assert_equal(doc4['Extended Summary'], [])
+    assert doc4['Extended Summary'] == []
 
 
 doc5 = NumpyDocString(
@@ -668,17 +729,19 @@ doc5 = NumpyDocString(
 
 
 def test_raises():
-    assert_equal(len(doc5['Raises']), 1)
-    name,_,desc = doc5['Raises'][0]
-    assert_equal(name,'LinAlgException')
-    assert_equal(desc,['If array is singular.'])
+    assert len(doc5['Raises']) == 1
+    param = doc5['Raises'][0]
+    assert param.name == ''
+    assert param.type == 'LinAlgException'
+    assert param.desc == ['If array is singular.']
 
 
 def test_warns():
-    assert_equal(len(doc5['Warns']), 1)
-    name,_,desc = doc5['Warns'][0]
-    assert_equal(name,'SomeWarning')
-    assert_equal(desc,['If needed'])
+    assert len(doc5['Warns']) == 1
+    param = doc5['Warns'][0]
+    assert param.name == ''
+    assert param.type == 'SomeWarning'
+    assert param.desc == ['If needed']
 
 
 def test_see_also():
@@ -694,36 +757,46 @@ def test_see_also():
              multiple lines
     func_f, func_g, :meth:`func_h`, func_j,
     func_k
+    func_f1, func_g1, :meth:`func_h1`, func_j1
+    func_f2, func_g2, :meth:`func_h2`, func_j2 : description of multiple
     :obj:`baz.obj_q`
     :obj:`~baz.obj_r`
     :class:`class_j`: fubar
         foobar
     """)
 
-    assert len(doc6['See Also']) == 13
-    for func, desc, role in doc6['See Also']:
-        if func in ('func_a', 'func_b', 'func_c', 'func_f',
-                    'func_g', 'func_h', 'func_j', 'func_k', 'baz.obj_q',
-                    '~baz.obj_r'):
-            assert(not desc)
-        else:
-            assert(desc)
+    assert len(doc6['See Also']) == 10
+    for funcs, desc in doc6['See Also']:
+        for func, role in funcs:
+            if func in ('func_a', 'func_b', 'func_c', 'func_f',
+                        'func_g', 'func_h', 'func_j', 'func_k', 'baz.obj_q',
+                        'func_f1', 'func_g1', 'func_h1', 'func_j1',
+                        '~baz.obj_r'):
+                assert not desc, str([func, desc])
+            elif func in ('func_f2', 'func_g2', 'func_h2', 'func_j2'):
+                assert desc, str([func, desc])
+            else:
+                assert desc, str([func, desc])
 
-        if func == 'func_h':
-            assert role == 'meth'
-        elif func == 'baz.obj_q' or func == '~baz.obj_r':
-            assert role == 'obj'
-        elif func == 'class_j':
-            assert role == 'class'
-        else:
-            assert role is None
+            if func == 'func_h':
+                assert role == 'meth'
+            elif func == 'baz.obj_q' or func == '~baz.obj_r':
+                assert role == 'obj'
+            elif func == 'class_j':
+                assert role == 'class'
+            elif func in ['func_h1', 'func_h2']:
+                assert role == 'meth'
+            else:
+                assert role is None, str([func, role])
 
-        if func == 'func_d':
-            assert desc == ['some equivalent func']
-        elif func == 'foo.func_e':
-            assert desc == ['some other func over', 'multiple lines']
-        elif func == 'class_j':
-            assert desc == ['fubar', 'foobar']
+            if func == 'func_d':
+                assert desc == ['some equivalent func']
+            elif func == 'foo.func_e':
+                assert desc == ['some other func over', 'multiple lines']
+            elif func == 'class_j':
+                assert desc == ['fubar', 'foobar']
+            elif func in ['func_f2', 'func_g2', 'func_h2', 'func_j2']:
+                assert desc == ['description of multiple'], str([desc, ['description of multiple']])
 
 
 def test_see_also_parse_error():
@@ -737,10 +810,11 @@ def test_see_also_parse_error():
     """)
     with assert_raises(ParseError) as err:
         NumpyDocString(text)
-    assert_equal(
-        str(r":func:`~foo` is not a item name in '\n    z(x,theta)\n\n    See Also\n    --------\n    :func:`~foo`\n    '"),
-        str(err.exception)
-    )
+
+    s1 = str(r":func:`~foo` is not a item name in '\n    z(x,theta)\n\n    See Also\n    --------\n    :func:`~foo`\n    '")
+    s2 = str(err.value)
+    assert s1 == s2
+
 
 def test_see_also_print():
     class Dummy(object):
@@ -754,13 +828,27 @@ def test_see_also_print():
         """
         pass
 
-    obj = Dummy()
-    s = str(FunctionDoc(obj, role='func'))
+    s = str(FunctionDoc(Dummy, role='func'))
     assert(':func:`func_a`, :func:`func_b`' in s)
     assert('    some relationship' in s)
     assert(':func:`func_d`' in s)
 
 
+def test_see_also_trailing_comma_warning():
+    warnings.filterwarnings('error')
+    with assert_warns(Warning, match='Unexpected comma or period after function list at index 43 of line .*'):
+        NumpyDocString(
+            """
+            z(x,theta)
+
+            See Also
+            --------
+            func_f2, func_g2, :meth:`func_h2`, func_j2, : description of multiple
+            :class:`class_j`: fubar
+                foobar
+            """)
+
+
 def test_unknown_section():
     doc_text = """
 Test having an unknown section
@@ -780,16 +868,18 @@ This should be ignored and warned about
         pass
 
     with warnings.catch_warnings(record=True) as w:
+        warnings.filterwarnings('always', '', UserWarning)
         NumpyDocString(doc_text)
         assert len(w) == 1
         assert "Unknown section Mope" == str(w[0].message)
 
     with warnings.catch_warnings(record=True) as w:
+        warnings.filterwarnings('always', '', UserWarning)
         SphinxClassDoc(BadSection)
         assert len(w) == 1
-        assert_true('test_docscrape.test_unknown_section.<locals>.BadSection'
-                    in str(w[0].message)
-                    or 'test_docscrape.BadSection' in str(w[0].message))
+        assert('test_docscrape.test_unknown_section.<locals>.BadSection'
+               in str(w[0].message)
+               or 'test_docscrape.BadSection' in str(w[0].message))
 
 
 doc7 = NumpyDocString("""
@@ -897,7 +987,7 @@ def test_use_blockquotes():
 
             GHI
 
-        **JKL**
+        JKL
 
             MNO
     ''')
@@ -1082,8 +1172,6 @@ def test_class_members_doc():
     b
     c
 
-    .. index::
-
     """)
 
 
@@ -1179,6 +1267,43 @@ def test_class_members_doc_sphinx():
     """)
 
 
+def test_class_attributes_as_member_list():
+
+    class Foo:
+        """
+        Class docstring.
+
+        Attributes
+        ----------
+        an_attribute
+            Another description that is not used.
+
+        """
+        @property
+        def an_attribute(self):
+            """Test attribute"""
+            return None
+
+    attr_doc = """:Attributes:
+
+    :obj:`an_attribute <an_attribute>`
+        Test attribute"""
+
+    assert attr_doc in str(SphinxClassDoc(Foo))
+    assert "Another description" not in str(SphinxClassDoc(Foo))
+
+    attr_doc2 = """.. rubric:: Attributes
+
+.. autosummary::
+   :toctree:
+
+   an_attribute"""
+
+    cfg = dict(attributes_as_param_list=False)
+    assert attr_doc2 in str(SphinxClassDoc(Foo, config=cfg))
+    assert "Another description" not in str(SphinxClassDoc(Foo, config=cfg))
+
+
 def test_templated_sections():
     doc = SphinxClassDoc(None, class_doc_txt,
                          config={'template': jinja2.Template('{{examples}}\n{{parameters}}')})
@@ -1226,6 +1351,139 @@ def test_nonstandard_property():
     assert "test attribute" in str(doc)
 
 
+def test_args_and_kwargs():
+    cfg = dict()
+    doc = SphinxDocString("""
+    Parameters
+    ----------
+    param1 : int
+        First parameter
+    *args : tuple
+        Arguments
+    **kwargs : dict
+        Keyword arguments
+    """, config=cfg)
+    line_by_line_compare(str(doc), r"""
+:Parameters:
+
+    **param1** : int
+        First parameter
+
+    **\*args** : tuple
+        Arguments
+
+    **\*\*kwargs** : dict
+        Keyword arguments
+    """)
+
+def test_autoclass():
+    cfg=dict(show_class_members=True,
+             show_inherited_class_members=True)
+    doc = SphinxClassDoc(str, '''
+A top section before
+
+.. autoclass:: str
+    ''', config=cfg)
+    line_by_line_compare(str(doc), r'''
+A top section before
+
+.. autoclass:: str
+
+.. rubric:: Methods
+
+
+    ''', 5)
+
+
+xref_doc_txt = """
+Test xref in Parameters, Other Parameters and Returns
+
+Parameters
+----------
+p1 : int
+    Integer value
+
+p2 : float, optional
+    Integer value
+
+Other Parameters
+----------------
+p3 : list[int]
+    List of integers
+p4 : :class:`pandas.DataFrame`
+    A dataframe
+p5 : sequence of `int`
+    A sequence
+
+Returns
+-------
+out : array
+    Numerical return value
+"""
+
+
+xref_doc_txt_expected = r"""
+Test xref in Parameters, Other Parameters and Returns
+
+
+:Parameters:
+
+    **p1** : :class:`python:int`
+        Integer value
+
+    **p2** : :class:`python:float`, optional
+        Integer value
+
+:Returns:
+
+    **out** : :obj:`array <numpy.ndarray>`
+        Numerical return value
+
+
+:Other Parameters:
+
+    **p3** : :class:`python:list`\[:class:`python:int`]
+        List of integers
+
+    **p4** : :class:`pandas.DataFrame`
+        A dataframe
+
+    **p5** : :obj:`python:sequence` of `int`
+        A sequence
+"""
+
+
+def test_xref():
+    xref_aliases = {
+        'sequence': ':obj:`python:sequence`',
+    }
+
+    class Config():
+        def __init__(self, a, b):
+            self.numpydoc_xref_aliases = a
+            self.numpydoc_xref_aliases_complete = b
+
+    xref_aliases_complete = deepcopy(DEFAULT_LINKS)
+    for key in xref_aliases:
+        xref_aliases_complete[key] = xref_aliases[key]
+    config = Config(xref_aliases, xref_aliases_complete)
+    app = namedtuple('config', 'config')(config)
+    update_config(app)
+
+    xref_ignore = {'of', 'default', 'optional'}
+
+    doc = SphinxDocString(
+        xref_doc_txt,
+        config=dict(
+            xref_param_type=True,
+            xref_aliases=xref_aliases_complete,
+            xref_ignore=xref_ignore
+        )
+    )
+
+    line_by_line_compare(str(doc), xref_doc_txt_expected)
+
+
 if __name__ == "__main__":
-    import nose
-    nose.run()
+    import pytest
+    pytest.main()
