@@ -12,6 +12,7 @@ Examples::
     $ python runtests.py --ipython
     $ python runtests.py --python somescript.py
     $ python runtests.py --bench
+    $ python runtests.py --no-build --bench signal.LTI
 
 Run a debugger:
 
@@ -49,12 +50,18 @@ else:
 
 import sys
 import os
+# the following multiprocessing import is necessary to prevent tests that use
+# multiprocessing from hanging on >= Python3.8 (macOS) using pytest. Just the
+# import is enough...
+import multiprocessing
+
 
 # In case we are run from the source directory, we don't want to import the
 # project from there:
 sys.path.pop(0)
 
 from argparse import ArgumentParser, REMAINDER
+import contextlib
 import shutil
 import subprocess
 import time
@@ -108,8 +115,7 @@ def main(argv):
     parser.add_argument("--debug", "-g", action="store_true",
                         help="Debug build")
     parser.add_argument("--parallel", "-j", type=int, default=1,
-                        help="Number of parallel jobs during build (requires "
-                             "NumPy 1.10 or greater).")
+                        help="Number of parallel jobs for build and testing")
     parser.add_argument("--show-build-log", action="store_true",
                         help="Show build output rather than using a log file")
     parser.add_argument("--bench", action="store_true",
@@ -123,8 +129,26 @@ def main(argv):
                               ))
     parser.add_argument("args", metavar="ARGS", default=[], nargs=REMAINDER,
                         help="Arguments to pass to Nose, Python or shell")
+    parser.add_argument("--pep8", action="store_true", default=False,
+                        help="Perform pep8 check with pycodestyle.")
+    parser.add_argument("--mypy", action="store_true", default=False,
+                        help="Run mypy on the codebase")
+    parser.add_argument("--doc", action="append", nargs="?",
+                        const="html-scipyorg", help="Build documentation")
     args = parser.parse_args(argv)
 
+    if args.pep8:
+        # Lint the source using the configuration in tox.ini.
+        os.system("pycodestyle scipy benchmarks/benchmarks")
+        # Lint just the diff since branching off of master using a
+        # stricter configuration.
+        lint_diff = os.path.join(ROOT_DIR, 'tools', 'lint_diff.py')
+        os.system(lint_diff)
+        sys.exit(0)
+
+    if args.mypy:
+        sys.exit(run_mypy(args))
+
     if args.bench_compare:
         args.bench = True
         args.no_build = True  # ASV does the building
@@ -164,7 +188,7 @@ def main(argv):
             sys.modules['__main__'] = new_module('__main__')
             ns = dict(__name__='__main__',
                       __file__=extra_argv[0])
-            exec_(script, ns)
+            exec(script, ns)
             sys.exit(0)
         else:
             import code
@@ -182,6 +206,14 @@ def main(argv):
         os.execv(shell, [shell] + extra_argv)
         sys.exit(1)
 
+    if args.doc:
+        cmd = ["make", "-Cdoc", 'PYTHON="{}"'.format(sys.executable)]
+        cmd += args.doc
+        if args.parallel:
+            cmd.append('SPHINXOPTS="-j{}"'.format(args.parallel))
+        subprocess.run(cmd, check=True)
+        sys.exit(0)
+
     if args.coverage:
         dst_dir = os.path.join(ROOT_DIR, 'build', 'coverage')
         fn = os.path.join(dst_dir, 'coverage_html.js')
@@ -210,10 +242,13 @@ def main(argv):
             bench_args.extend(['--bench', a])
 
         if not args.bench_compare:
-            cmd = [os.path.join(ROOT_DIR, 'benchmarks', 'run.py'),
-                   'run', '-n', '-e', '--python=same'] + bench_args
-            os.execv(sys.executable, [sys.executable] + cmd)
-            sys.exit(1)
+            import scipy
+            print("Running benchmarks for Scipy version %s at %s"
+                  % (scipy.__version__, scipy.__file__))
+            cmd = ['asv', 'run', '--dry-run', '--show-stderr',
+                   '--python=same'] + bench_args
+            retval = run_asv(cmd)
+            sys.exit(retval)
         else:
             if len(args.bench_compare) == 1:
                 commit_a = args.bench_compare[0]
@@ -245,10 +280,9 @@ def main(argv):
             out, err = p.communicate()
             commit_a = out.strip()
 
-            cmd = [os.path.join(ROOT_DIR, 'benchmarks', 'run.py'),
-                   'continuous', '-e', '-f', '1.05',
+            cmd = ['asv', 'continuous', '--show-stderr', '--factor', '1.05',
                    commit_a, commit_b] + bench_args
-            os.execv(sys.executable, [sys.executable] + cmd)
+            run_asv(cmd)
             sys.exit(1)
 
     if args.build_only:
@@ -284,7 +318,8 @@ def main(argv):
                       extra_argv=extra_argv,
                       doctests=args.doctests,
                       coverage=args.coverage,
-                      tests=tests)
+                      tests=tests,
+                      parallel=args.parallel)
     finally:
         os.chdir(cwd)
 
@@ -332,8 +367,8 @@ def build_project(args):
             cvars = distutils.sysconfig.get_config_vars()
             env['OPT'] = '-O0 -ggdb'
             env['FOPT'] = '-O0 -ggdb'
-            env['CC'] = cvars['CC'] + ' --coverage'
-            env['CXX'] = cvars['CXX'] + ' --coverage'
+            env['CC'] = env.get('CC', cvars['CC']) + ' --coverage'
+            env['CXX'] = env.get('CXX', cvars['CXX']) + ' --coverage'
             env['F77'] = 'gfortran --coverage '
             env['F90'] = 'gfortran --coverage '
             env['LDSHARED'] = cvars['LDSHARED'] + ' --coverage'
@@ -343,7 +378,7 @@ def build_project(args):
     cmd += ['build']
     if args.parallel > 1:
         cmd += ['-j', str(args.parallel)]
-    # Install; avoid producing eggs so scipy can be imported from dst_dir.
+    # Install; avoid producing eggs so SciPy can be imported from dst_dir.
     cmd += ['install', '--prefix=' + dst_dir,
             '--single-version-externally-managed',
             '--record=' + dst_dir + 'tmp_install_log.txt']
@@ -351,7 +386,7 @@ def build_project(args):
     from distutils.sysconfig import get_python_lib
     site_dir = get_python_lib(prefix=dst_dir, plat_specific=True)
     # easy_install won't install to a path that Python by default cannot see
-    # and isn't on the PYTHONPATH.  Plus, it has to exist.
+    # and isn't on the PYTHONPATH. Plus, it has to exist.
     if not os.path.exists(site_dir):
         os.makedirs(site_dir)
     env['PYTHONPATH'] = site_dir
@@ -452,25 +487,79 @@ def lcov_generate():
         print("HTML output generated under build/lcov/")
 
 
-#
-# Python 3 support
-#
+@contextlib.contextmanager
+def working_dir(new_dir):
+    current_dir = os.getcwd()
+    try:
+        os.chdir(new_dir)
+        yield
+    finally:
+        os.chdir(current_dir)
+
+
+def run_mypy(args):
+    if args.no_build:
+        raise ValueError('Cannot run mypy with --no-build')
+
+    try:
+        import mypy.api
+    except ImportError as e:
+        raise RuntimeError(
+            "Mypy not found. Please install it by running "
+            "pip install -r mypy_requirements.txt from the repo root"
+        ) from e
+
+    site_dir = build_project(args)
+    config = os.path.join(
+        os.path.dirname(os.path.abspath(__file__)),
+        "mypy.ini",
+    )
+    with working_dir(site_dir):
+        # By default mypy won't color the output since it isn't being
+        # invoked from a tty.
+        os.environ['MYPY_FORCE_COLOR'] = '1'
+        # Change to the site directory to make sure mypy doesn't pick
+        # up any type stubs in the source tree.
+        report, errors, status = mypy.api.run([
+            "--config-file",
+            config,
+            PROJECT_MODULE,
+        ])
+    print(report, end='')
+    print(errors, end='', file=sys.stderr)
+    return status
+
+
+def run_asv(cmd):
+    cwd = os.path.join(os.path.abspath(os.path.dirname(__file__)),
+                       'benchmarks')
+    # Always use ccache, if installed
+    env = dict(os.environ)
+    env['PATH'] = os.pathsep.join(EXTRA_PATH +
+                                  env.get('PATH', '').split(os.pathsep))
+    # Control BLAS/LAPACK threads
+    env['OPENBLAS_NUM_THREADS'] = '1'
+    env['MKL_NUM_THREADS'] = '1'
+
+    # Limit memory usage
+    sys.path.insert(0, cwd)
+    from benchmarks.common import set_mem_rlimit
+    try:
+        set_mem_rlimit()
+    except (ImportError, RuntimeError):
+        pass
+
+    # Run
+    try:
+        return subprocess.call(cmd, env=env, cwd=cwd)
+    except OSError as err:
+        if err.errno == errno.ENOENT:
+            print("Error when running '%s': %s\n" % (" ".join(cmd), str(err),))
+            print("You need to install Airspeed Velocity (https://airspeed-velocity.github.io/asv/)")
+            print("to run Scipy benchmarks")
+            return 1
+        raise
 
-if sys.version_info[0] >= 3:
-    import builtins
-    exec_ = getattr(builtins, "exec")
-else:
-    def exec_(code, globs=None, locs=None):
-        """Execute code in a namespace."""
-        if globs is None:
-            frame = sys._getframe(1)
-            globs = frame.f_globals
-            if locs is None:
-                locs = frame.f_locals
-            del frame
-        elif locs is None:
-            locs = globs
-        exec("""exec code in globs, locs""")
 
 if __name__ == "__main__":
     main(argv=sys.argv[1:])
