@@ -11,13 +11,10 @@ Functions
     scalar_search_wolfe2
 
 """
-from __future__ import division, print_function, absolute_import
-
 from warnings import warn
 
 from scipy.optimize import minpack2
 import numpy as np
-from scipy._lib.six import xrange
 
 __all__ = ['LineSearchWarning', 'line_search_wolfe1', 'line_search_wolfe2',
            'scalar_search_wolfe1', 'scalar_search_wolfe2',
@@ -166,7 +163,7 @@ def scalar_search_wolfe1(phi, derphi, phi0=None, old_phi0=None, derphi0=None,
     task = b'START'
 
     maxiter = 100
-    for i in xrange(maxiter):
+    for i in range(maxiter):
         stp, phi1, derphi1, task = minpack2.dcsrch(alpha1, phi1, derphi1,
                                                    c1, c2, xtol, task,
                                                    amin, amax, isave, dsave)
@@ -214,7 +211,7 @@ def line_search_wolfe2(f, myfprime, xk, pk, gfk=None, old_fval=None,
     old_fval : float, optional
         Function value for x=xk. Will be recomputed if omitted.
     old_old_fval : float, optional
-        Function value for the point preceding x=xk
+        Function value for the point preceding x=xk.
     args : tuple, optional
         Additional arguments passed to objective function.
     c1 : float, optional
@@ -226,14 +223,14 @@ def line_search_wolfe2(f, myfprime, xk, pk, gfk=None, old_fval=None,
     extra_condition : callable, optional
         A callable of the form ``extra_condition(alpha, x, f, g)``
         returning a boolean. Arguments are the proposed step ``alpha``
-        and the corresponding ``x``, ``f`` and ``g`` values. The line search 
-        accepts the value of ``alpha`` only if this 
-        callable returns ``True``. If the callable returns ``False`` 
-        for the step length, the algorithm will continue with 
-        new iterates. The callable is only called for iterates 
+        and the corresponding ``x``, ``f`` and ``g`` values. The line search
+        accepts the value of ``alpha`` only if this
+        callable returns ``True``. If the callable returns ``False``
+        for the step length, the algorithm will continue with
+        new iterates. The callable is only called for iterates
         satisfying the strong Wolfe conditions.
     maxiter : int, optional
-        Maximum number of iterations to perform
+        Maximum number of iterations to perform.
 
     Returns
     -------
@@ -258,10 +255,26 @@ def line_search_wolfe2(f, myfprime, xk, pk, gfk=None, old_fval=None,
     Notes
     -----
     Uses the line search algorithm to enforce strong Wolfe
-    conditions.  See Wright and Nocedal, 'Numerical Optimization',
-    1999, pg. 59-60.
+    conditions. See Wright and Nocedal, 'Numerical Optimization',
+    1999, pp. 59-61.
 
-    For the zoom phase it uses an algorithm by [...].
+    Examples
+    --------
+    >>> from scipy.optimize import line_search
+
+    A objective function and its gradient are defined.
+
+    >>> def obj_func(x):
+    ...     return (x[0])**2+(x[1])**2
+    >>> def obj_grad(x):
+    ...     return [2*x[0], 2*x[1]]
+
+    We can find alpha that satisfies strong Wolfe conditions.
+
+    >>> start_point = np.array([1.8, 1.7])
+    >>> search_gradient = np.array([-1.0, -1.0])
+    >>> line_search(obj_func, obj_grad, start_point, search_gradient)
+    (1.0, 2, 1, 1.1300000000000001, 6.13, [1.6, 1.4])
 
     """
     fc = [0]
@@ -337,9 +350,9 @@ def scalar_search_wolfe2(phi, derphi, phi0=None,
     derphi : callable phi'(alpha)
         Objective function derivative. Returns a scalar.
     phi0 : float, optional
-        Value of phi at 0
+        Value of phi at 0.
     old_phi0 : float, optional
-        Value of phi at previous point
+        Value of phi at previous point.
     derphi0 : float, optional
         Value of derphi at 0
     c1 : float, optional
@@ -347,7 +360,7 @@ def scalar_search_wolfe2(phi, derphi, phi0=None,
     c2 : float, optional
         Parameter for curvature condition rule.
     amax : float, optional
-        Maximum step size
+        Maximum step size.
     extra_condition : callable, optional
         A callable of the form ``extra_condition(alpha, phi_value)``
         returning a boolean. The line search accepts the value
@@ -357,16 +370,16 @@ def scalar_search_wolfe2(phi, derphi, phi0=None,
         The callable is only called for iterates satisfying
         the strong Wolfe conditions.
     maxiter : int, optional
-        Maximum number of iterations to perform
+        Maximum number of iterations to perform.
 
     Returns
     -------
     alpha_star : float or None
         Best alpha, or None if the line search algorithm did not converge.
     phi_star : float
-        phi at alpha_star
+        phi at alpha_star.
     phi0 : float
-        phi at 0
+        phi at 0.
     derphi_star : float or None
         derphi at alpha_star, or None if the line search algorithm
         did not converge.
@@ -374,10 +387,8 @@ def scalar_search_wolfe2(phi, derphi, phi0=None,
     Notes
     -----
     Uses the line search algorithm to enforce strong Wolfe
-    conditions.  See Wright and Nocedal, 'Numerical Optimization',
-    1999, pg. 59-60.
-
-    For the zoom phase it uses an algorithm by [...].
+    conditions. See Wright and Nocedal, 'Numerical Optimization',
+    1999, pp. 59-61.
 
     """
 
@@ -400,7 +411,7 @@ def scalar_search_wolfe2(phi, derphi, phi0=None,
         alpha1 = min(alpha1, amax)
 
     phi_a1 = phi(alpha1)
-    #derphi_a1 = derphi(alpha1)  evaluated below
+    #derphi_a1 = derphi(alpha1) evaluated below
 
     phi_a0 = phi0
     derphi_a0 = derphi0
@@ -408,7 +419,7 @@ def scalar_search_wolfe2(phi, derphi, phi0=None,
     if extra_condition is None:
         extra_condition = lambda alpha, phi: True
 
-    for i in xrange(maxiter):
+    for i in range(maxiter):
         if alpha1 == 0 or (amax is not None and alpha0 == amax):
             # alpha1 == 0: This shouldn't happen. Perhaps the increment has
             # slipped below machine precision?
@@ -426,8 +437,9 @@ def scalar_search_wolfe2(phi, derphi, phi0=None,
             warn(msg, LineSearchWarning)
             break
 
+        not_first_iteration = i > 0
         if (phi_a1 > phi0 + c1 * alpha1 * derphi0) or \
-           ((phi_a1 >= phi_a0) and (i > 1)):
+           ((phi_a1 >= phi_a0) and not_first_iteration):
             alpha_star, phi_star, derphi_star = \
                         _zoom(alpha0, alpha1, phi_a0,
                               phi_a1, derphi_a0, phi, derphi,
@@ -473,7 +485,7 @@ def _cubicmin(a, fa, fpa, b, fb, c, fc):
     Finds the minimizer for a cubic polynomial that goes through the
     points (a,fa), (b,fb), and (c,fc) with derivative at a of fpa.
 
-    If no minimizer can be found return None
+    If no minimizer can be found, return None.
 
     """
     # f(x) = A *(x-a)^3 + B*(x-a)^2 + C*(x-a) + D
@@ -505,7 +517,7 @@ def _cubicmin(a, fa, fpa, b, fb, c, fc):
 def _quadmin(a, fa, fpa, b, fb):
     """
     Finds the minimizer for a quadratic polynomial that goes through
-    the points (a,fa), (b,fb) with derivative at a of fpa,
+    the points (a,fa), (b,fb) with derivative at a of fpa.
 
     """
     # f(x) = B*(x-a)^2 + C*(x-a) + D
@@ -525,8 +537,15 @@ def _quadmin(a, fa, fpa, b, fb):
 
 def _zoom(a_lo, a_hi, phi_lo, phi_hi, derphi_lo,
           phi, derphi, phi0, derphi0, c1, c2, extra_condition):
-    """
+    """Zoom stage of approximate linesearch satisfying strong Wolfe conditions.
+    
     Part of the optimization algorithm in `scalar_search_wolfe2`.
+    
+    Notes
+    -----
+    Implements Algorithm 3.6 (zoom) in Wright and Nocedal,
+    'Numerical Optimization', 1999, pp. 61.
+
     """
 
     maxiter = 10
@@ -537,7 +556,7 @@ def _zoom(a_lo, a_hi, phi_lo, phi_hi, derphi_lo,
     a_rec = 0
     while True:
         # interpolate to find a trial step length between a_lo and
-        # a_hi Need to choose interpolation here.  Use cubic
+        # a_hi Need to choose interpolation here. Use cubic
         # interpolation and then if the result is within delta *
         # dalpha or outside of the interval bounded by a_lo or a_hi
         # then use quadratic interpolation, if the result is still too
@@ -553,7 +572,7 @@ def _zoom(a_lo, a_hi, phi_lo, phi_hi, derphi_lo,
         # (uses phi_lo, derphi_lo, phi_hi, and the most recent value of phi)
         #
         # if the result is too close to the end points (or out of the
-        # interval) then use quadratic interpolation with phi_lo,
+        # interval), then use quadratic interpolation with phi_lo,
         # derphi_lo and phi_hi if the result is still too close to the
         # end points (or out of the interval) then use bisection
 
@@ -638,7 +657,7 @@ def line_search_armijo(f, xk, pk, gfk, old_fval, args=(), c1=1e-4, alpha0=1):
     Notes
     -----
     Uses the interpolation algorithm (Armijo backtracking) as suggested by
-    Wright and Nocedal in 'Numerical Optimization', 1999, pg. 56-57
+    Wright and Nocedal in 'Numerical Optimization', 1999, pp. 56-57
 
     """
     xk = np.atleast_1d(xk)
@@ -672,7 +691,7 @@ def scalar_search_armijo(phi, phi0, derphi0, c1=1e-4, alpha0=1, amin=0):
     """Minimize over alpha, the function ``phi(alpha)``.
 
     Uses the interpolation algorithm (Armijo backtracking) as suggested by
-    Wright and Nocedal in 'Numerical Optimization', 1999, pg. 56-57
+    Wright and Nocedal in 'Numerical Optimization', 1999, pp. 56-57
 
     alpha > 0 is assumed to be a descent direction.
 
@@ -686,7 +705,7 @@ def scalar_search_armijo(phi, phi0, derphi0, c1=1e-4, alpha0=1, amin=0):
     if phi_a0 <= phi0 + c1*alpha0*derphi0:
         return alpha0, phi_a0
 
-    # Otherwise compute the minimizer of a quadratic interpolant:
+    # Otherwise, compute the minimizer of a quadratic interpolant:
 
     alpha1 = -(derphi0) * alpha0**2 / 2.0 / (phi_a0 - phi0 - derphi0 * alpha0)
     phi_a1 = phi(alpha1)
@@ -694,7 +713,7 @@ def scalar_search_armijo(phi, phi0, derphi0, c1=1e-4, alpha0=1, amin=0):
     if (phi_a1 <= phi0 + c1*alpha1*derphi0):
         return alpha1, phi_a1
 
-    # Otherwise loop with cubic interpolation until we find an alpha which
+    # Otherwise, loop with cubic interpolation until we find an alpha which
     # satisfies the first Wolfe condition (since we are backtracking, we will
     # assume that the value of alpha is not too small and satisfies the second
     # condition.
@@ -741,9 +760,9 @@ def _nonmonotone_line_search_cruz(f, x_k, d, prev_fs, eta,
         Function returning a tuple ``(f, F)`` where ``f`` is the value
         of a merit function and ``F`` the residual.
     x_k : ndarray
-        Initial position
+        Initial position.
     d : ndarray
-        Search direction
+        Search direction.
     prev_fs : float
         List of previous merit function values. Should have ``len(prev_fs) <= M``
         where ``M`` is the nonmonotonicity window parameter.
@@ -814,11 +833,11 @@ def _nonmonotone_line_search_cheng(f, x_k, d, f_k, C, Q, eta,
         Function returning a tuple ``(f, F)`` where ``f`` is the value
         of a merit function and ``F`` the residual.
     x_k : ndarray
-        Initial position
+        Initial position.
     d : ndarray
-        Search direction
+        Search direction.
     f_k : float
-        Initial merit function value
+        Initial merit function value.
     C, Q : float
         Control parameters. On the first iteration, give values
         Q=1.0, C=f_k
