@@ -1,17 +1,14 @@
 """Tools for spectral analysis.
 """
 
-from __future__ import division, print_function, absolute_import
-
 import numpy as np
-from scipy import fftpack
+from scipy import fft as sp_fft
 from . import signaltools
 from .windows import get_window
 from ._spectral import _lombscargle
 from ._arraytools import const_ext, even_ext, odd_ext, zero_ext
 import warnings
 
-from scipy._lib.six import string_types
 
 __all__ = ['periodogram', 'welch', 'lombscargle', 'csd', 'coherence',
            'spectrogram', 'stft', 'istft', 'check_COLA', 'check_NOLA']
@@ -38,7 +35,7 @@ def lombscargle(x,
     When *normalize* is True the computed periodogram is normalized by
     the residuals of the data around a constant reference model (at zero).
 
-    Input arrays should be one-dimensional and will be cast to float64.
+    Input arrays should be 1-D and will be cast to float64.
 
     Parameters
     ----------
@@ -49,7 +46,7 @@ def lombscargle(x,
     freqs : array_like
         Angular frequencies for output periodogram.
     precenter : bool, optional
-        Pre-center amplitudes by subtracting the mean.
+        Pre-center measurement values by subtracting the mean.
     normalize : bool, optional
         Compute normalized periodogram.
 
@@ -187,7 +184,7 @@ def periodogram(x, fs=1.0, window='boxcar', nfft=None, detrend='constant',
         done. Defaults to 'constant'.
     return_onesided : bool, optional
         If `True`, return a one-sided spectrum for real data. If
-        `False` return a two-sided spectrum. Defaults to `True`, but for 
+        `False` return a two-sided spectrum. Defaults to `True`, but for
         complex data, a two-sided spectrum is always returned.
     scaling : { 'density', 'spectrum' }, optional
         Selects between computing the power spectral density ('density')
@@ -333,7 +330,7 @@ def welch(x, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
         done. Defaults to 'constant'.
     return_onesided : bool, optional
         If `True`, return a one-sided spectrum for real data. If
-        `False` return a two-sided spectrum. Defaults to `True`, but for 
+        `False` return a two-sided spectrum. Defaults to `True`, but for
         complex data, a two-sided spectrum is always returned.
     scaling : { 'density', 'spectrum' }, optional
         Selects between computing the power spectral density ('density')
@@ -498,7 +495,7 @@ def csd(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
         done. Defaults to 'constant'.
     return_onesided : bool, optional
         If `True`, return a one-sided spectrum for real data. If
-        `False` return a two-sided spectrum. Defaults to `True`, but for 
+        `False` return a two-sided spectrum. Defaults to `True`, but for
         complex data, a two-sided spectrum is always returned.
     scaling : { 'density', 'spectrum' }, optional
         Selects between computing the cross spectral density ('density')
@@ -641,7 +638,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
         done. Defaults to 'constant'.
     return_onesided : bool, optional
         If `True`, return a one-sided spectrum for real data. If
-        `False` return a two-sided spectrum. Defaults to `True`, but for 
+        `False` return a two-sided spectrum. Defaults to `True`, but for
         complex data, a two-sided spectrum is always returned.
     scaling : { 'density', 'spectrum' }, optional
         Selects between computing the power spectral density ('density')
@@ -697,6 +694,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
     Examples
     --------
     >>> from scipy import signal
+    >>> from scipy.fft import fftshift
     >>> import matplotlib.pyplot as plt
 
     Generate a test signal, a 2 Vrms sine wave whose frequency is slowly
@@ -717,7 +715,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
     Compute and plot the spectrogram.
 
     >>> f, t, Sxx = signal.spectrogram(x, fs)
-    >>> plt.pcolormesh(t, f, Sxx)
+    >>> plt.pcolormesh(t, f, Sxx, shading='gouraud')
     >>> plt.ylabel('Frequency [Hz]')
     >>> plt.xlabel('Time [sec]')
     >>> plt.show()
@@ -725,7 +723,7 @@ def spectrogram(x, fs=1.0, window=('tukey', .25), nperseg=None, noverlap=None,
     Note, if using output that is not one sided, then use the following:
 
     >>> f, t, Sxx = signal.spectrogram(x, fs, return_onesided=False)
-    >>> plt.pcolormesh(t, np.fft.fftshift(f), np.fft.fftshift(Sxx, axes=0))
+    >>> plt.pcolormesh(t, fftshift(f), fftshift(Sxx, axes=0), shading='gouraud')
     >>> plt.ylabel('Frequency [Hz]')
     >>> plt.xlabel('Time [sec]')
     >>> plt.show()
@@ -839,29 +837,29 @@ def check_COLA(window, nperseg, noverlap, tol=1e-10):
 
     Confirm COLA condition for rectangular window of 75% (3/4) overlap:
 
-    >>> signal.check_COLA(signal.boxcar(100), 100, 75)
+    >>> signal.check_COLA(signal.windows.boxcar(100), 100, 75)
     True
 
     COLA is not true for 25% (1/4) overlap, though:
 
-    >>> signal.check_COLA(signal.boxcar(100), 100, 25)
+    >>> signal.check_COLA(signal.windows.boxcar(100), 100, 25)
     False
 
     "Symmetrical" Hann window (for filter design) is not COLA:
 
-    >>> signal.check_COLA(signal.hann(120, sym=True), 120, 60)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=True), 120, 60)
     False
 
     "Periodic" or "DFT-even" Hann window (for FFT analysis) is COLA for
     overlap of 1/2, 2/3, 3/4, etc.:
 
-    >>> signal.check_COLA(signal.hann(120, sym=False), 120, 60)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=False), 120, 60)
     True
 
-    >>> signal.check_COLA(signal.hann(120, sym=False), 120, 80)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=False), 120, 80)
     True
 
-    >>> signal.check_COLA(signal.hann(120, sym=False), 120, 90)
+    >>> signal.check_COLA(signal.windows.hann(120, sym=False), 120, 90)
     True
 
     """
@@ -875,7 +873,7 @@ def check_COLA(window, nperseg, noverlap, tol=1e-10):
         raise ValueError('noverlap must be less than nperseg.')
     noverlap = int(noverlap)
 
-    if isinstance(window, string_types) or type(window) is tuple:
+    if isinstance(window, str) or type(window) is tuple:
         win = get_window(window, nperseg)
     else:
         win = np.asarray(window)
@@ -960,17 +958,17 @@ def check_NOLA(window, nperseg, noverlap, tol=1e-10):
 
     Confirm NOLA condition for rectangular window of 75% (3/4) overlap:
 
-    >>> signal.check_NOLA(signal.boxcar(100), 100, 75)
+    >>> signal.check_NOLA(signal.windows.boxcar(100), 100, 75)
     True
 
     NOLA is also true for 25% (1/4) overlap:
 
-    >>> signal.check_NOLA(signal.boxcar(100), 100, 25)
+    >>> signal.check_NOLA(signal.windows.boxcar(100), 100, 25)
     True
 
     "Symmetrical" Hann window (for filter design) is also NOLA:
 
-    >>> signal.check_NOLA(signal.hann(120, sym=True), 120, 60)
+    >>> signal.check_NOLA(signal.windows.hann(120, sym=True), 120, 60)
     True
 
     As long as there is overlap, it takes quite a pathological window to fail
@@ -984,11 +982,11 @@ def check_NOLA(window, nperseg, noverlap, tol=1e-10):
     If there is not enough overlap, a window with zeros at the ends will not
     work:
 
-    >>> signal.check_NOLA(signal.hann(64), 64, 0)
+    >>> signal.check_NOLA(signal.windows.hann(64), 64, 0)
     False
-    >>> signal.check_NOLA(signal.hann(64), 64, 1)
+    >>> signal.check_NOLA(signal.windows.hann(64), 64, 1)
     False
-    >>> signal.check_NOLA(signal.hann(64), 64, 2)
+    >>> signal.check_NOLA(signal.windows.hann(64), 64, 2)
     True
     """
 
@@ -1003,7 +1001,7 @@ def check_NOLA(window, nperseg, noverlap, tol=1e-10):
         raise ValueError('noverlap must be a nonnegative integer')
     noverlap = int(noverlap)
 
-    if isinstance(window, string_types) or type(window) is tuple:
+    if isinstance(window, str) or type(window) is tuple:
         win = get_window(window, nperseg)
     else:
         win = np.asarray(window)
@@ -1060,7 +1058,7 @@ def stft(x, fs=1.0, window='hann', nperseg=256, noverlap=None, nfft=None,
         done. Defaults to `False`.
     return_onesided : bool, optional
         If `True`, return a one-sided spectrum for real data. If
-        `False` return a two-sided spectrum. Defaults to `True`, but for 
+        `False` return a two-sided spectrum. Defaults to `True`, but for
         complex data, a two-sided spectrum is always returned.
     boundary : str or None, optional
         Specifies whether the input signal is extended at both ends, and
@@ -1161,7 +1159,7 @@ def stft(x, fs=1.0, window='hann', nperseg=256, noverlap=None, nfft=None,
     Compute and plot the STFT's magnitude.
 
     >>> f, t, Zxx = signal.stft(x, fs, nperseg=1000)
-    >>> plt.pcolormesh(t, f, np.abs(Zxx), vmin=0, vmax=amp)
+    >>> plt.pcolormesh(t, f, np.abs(Zxx), vmin=0, vmax=amp, shading='gouraud')
     >>> plt.title('STFT Magnitude')
     >>> plt.ylabel('Frequency [Hz]')
     >>> plt.xlabel('Time [sec]')
@@ -1306,7 +1304,7 @@ def istft(Zxx, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
 
     >>> f, t, Zxx = signal.stft(x, fs=fs, nperseg=nperseg)
     >>> plt.figure()
-    >>> plt.pcolormesh(t, f, np.abs(Zxx), vmin=0, vmax=amp)
+    >>> plt.pcolormesh(t, f, np.abs(Zxx), vmin=0, vmax=amp, shading='gouraud')
     >>> plt.ylim([f[1], f[-1]])
     >>> plt.title('STFT Magnitude')
     >>> plt.ylabel('Frequency [Hz]')
@@ -1402,7 +1400,7 @@ def istft(Zxx, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
         Zxx = np.transpose(Zxx, zouter+[freq_axis, time_axis])
 
     # Get window as array
-    if isinstance(window, string_types) or type(window) is tuple:
+    if isinstance(window, str) or type(window) is tuple:
         win = get_window(window, nperseg)
     else:
         win = np.asarray(window)
@@ -1411,11 +1409,7 @@ def istft(Zxx, fs=1.0, window='hann', nperseg=None, noverlap=None, nfft=None,
         if win.shape[0] != nperseg:
             raise ValueError('window must have length of {0}'.format(nperseg))
 
-    if input_onesided:
-        ifunc = np.fft.irfft
-    else:
-        ifunc = fftpack.ifft
-
+    ifunc = sp_fft.irfft if input_onesided else sp_fft.ifft
     xsubs = ifunc(Zxx, axis=-2, n=nfft)[..., :nperseg, :]
 
     # Initialize output and normalization arrays
@@ -1626,7 +1620,7 @@ def _spectral_helper(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
         done. Defaults to 'constant'.
     return_onesided : bool, optional
         If `True`, return a one-sided spectrum for real data. If
-        `False` return a two-sided spectrum. Defaults to `True`, but for 
+        `False` return a two-sided spectrum. Defaults to `True`, but for
         complex data, a two-sided spectrum is always returned.
     scaling : { 'density', 'spectrum' }, optional
         Selects between computing the cross spectral density ('density')
@@ -1707,8 +1701,8 @@ def _spectral_helper(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
         youter.pop(axis)
         try:
             outershape = np.broadcast(np.empty(xouter), np.empty(youter)).shape
-        except ValueError:
-            raise ValueError('x and y cannot be broadcast together.')
+        except ValueError as e:
+            raise ValueError('x and y cannot be broadcast together.') from e
 
     if same_data:
         if x.size == 0:
@@ -1828,9 +1822,9 @@ def _spectral_helper(x, y, fs=1.0, window='hann', nperseg=None, noverlap=None,
         sides = 'twosided'
 
     if sides == 'twosided':
-        freqs = fftpack.fftfreq(nfft, 1/fs)
+        freqs = sp_fft.fftfreq(nfft, 1/fs)
     elif sides == 'onesided':
-        freqs = np.fft.rfftfreq(nfft, 1/fs)
+        freqs = sp_fft.rfftfreq(nfft, 1/fs)
 
     # Perform the windowed FFTs
     result = _fft_helper(x, win, detrend_func, nperseg, noverlap, nfft, sides)
@@ -1914,10 +1908,10 @@ def _fft_helper(x, win, detrend_func, nperseg, noverlap, nfft, sides):
 
     # Perform the fft. Acts on last axis by default. Zero-pads automatically
     if sides == 'twosided':
-        func = fftpack.fft
+        func = sp_fft.fft
     else:
         result = result.real
-        func = np.fft.rfft
+        func = sp_fft.rfft
     result = func(result, n=nfft)
 
     return result
@@ -1959,7 +1953,7 @@ def _triage_segments(window, nperseg, input_length):
     """
 
     # parse window; if array like, then set nperseg = win.shape
-    if isinstance(window, string_types) or isinstance(window, tuple):
+    if isinstance(window, str) or isinstance(window, tuple):
         # if nperseg not specified
         if nperseg is None:
             nperseg = 256  # then change to default
@@ -1989,7 +1983,7 @@ def _median_bias(n):
     Returns the bias of the median of a set of periodograms relative to
     the mean.
 
-    See arXiv:gr-qc/0509116 Appendix B for details.
+    See Appendix B from [1]_ for details.
 
     Parameters
     ----------
@@ -2000,6 +1994,13 @@ def _median_bias(n):
     -------
     bias : float
         Calculated bias.
+
+    References
+    ----------
+    .. [1] B. Allen, W.G. Anderson, P.R. Brady, D.A. Brown, J.D.E. Creighton.
+           "FINDCHIRP: an algorithm for detection of gravitational waves from
+           inspiraling compact binaries", Physical Review D 85, 2012,
+           :arxiv:`gr-qc/0509116`
     """
     ii_2 = 2 * np.arange(1., (n-1) // 2 + 1)
     return 1 + np.sum(1. / (ii_2 + 1) - 1. / ii_2)
