@@ -34,6 +34,7 @@
 #include "numpy/ndarrayobject.h"
 
 #include "sparsetools.h"
+#include "util.h"
 
 #define MAX_ARGS 16
 
@@ -266,16 +267,7 @@ call_thunk(char ret_spec, const char *spec, thunk_t *thunk, PyObject *args)
             /* Integer scalars */
             PY_LONG_LONG value;
 
-#if PY_VERSION_HEX >= 0x03000000
             value = PyLong_AsLongLong(arg_arrays[j]);
-#else
-            if (PyInt_Check(arg_arrays[j])) {
-                value = PyInt_AsLong(arg_arrays[j]);
-            }
-            else {
-                value = PyLong_AsLongLong(arg_arrays[j]);
-            }
-#endif
             if (PyErr_Occurred()) {
                 goto fail;
             }
@@ -447,7 +439,7 @@ fail:
         }
         #ifdef HAVE_WRITEBACKIFCOPY
             if (is_output[j] && arg_arrays[j] != NULL && PyArray_Check(arg_arrays[j])) {
-                PyArray_ResolveWritebackIfCopy((PyArrayObject *)arg_arrays[j]); 
+                PyArray_ResolveWritebackIfCopy((PyArrayObject *)arg_arrays[j]);
             }
         #endif
         Py_XDECREF(arg_arrays[j]);
@@ -477,23 +469,7 @@ static void *allocate_std_vector_typenum(int typenum)
     }
 
     try {
-        PROCESS(NPY_BOOL, npy_bool_wrapper);
-        PROCESS(NPY_BYTE, npy_byte);
-        PROCESS(NPY_UBYTE, npy_ubyte);
-        PROCESS(NPY_SHORT, npy_short);
-        PROCESS(NPY_USHORT, npy_ushort);
-        PROCESS(NPY_INT, npy_int);
-        PROCESS(NPY_UINT, npy_uint);
-        PROCESS(NPY_LONG, npy_long);
-        PROCESS(NPY_ULONG, npy_ulong);
-        PROCESS(NPY_LONGLONG, npy_longlong);
-        PROCESS(NPY_ULONGLONG, npy_ulonglong);
-        PROCESS(NPY_FLOAT, npy_float);
-        PROCESS(NPY_DOUBLE, npy_double);
-        PROCESS(NPY_LONGDOUBLE, npy_longdouble);
-        PROCESS(NPY_CFLOAT, npy_cfloat_wrapper);
-        PROCESS(NPY_CDOUBLE, npy_cdouble_wrapper);
-        PROCESS(NPY_CLONGDOUBLE, npy_clongdouble_wrapper);
+        SPTOOLS_FOR_EACH_DATA_TYPE_CODE(PROCESS)
     } catch (std::exception &e) {
         /* failed */
     }
@@ -510,25 +486,10 @@ static void free_std_vector_typenum(int typenum, void *p)
 #define PROCESS(ntype, ctype)                                   \
     if (PyArray_EquivTypenums(typenum, ntype)) {                \
         delete ((std::vector<ctype>*)p);                        \
+        return;                                                 \
     }
 
-    PROCESS(NPY_BOOL, npy_bool_wrapper);
-    PROCESS(NPY_BYTE, npy_byte);
-    PROCESS(NPY_UBYTE, npy_ubyte);
-    PROCESS(NPY_SHORT, npy_short);
-    PROCESS(NPY_USHORT, npy_ushort);
-    PROCESS(NPY_INT, npy_int);
-    PROCESS(NPY_UINT, npy_uint);
-    PROCESS(NPY_LONG, npy_long);
-    PROCESS(NPY_ULONG, npy_ulong);
-    PROCESS(NPY_LONGLONG, npy_longlong);
-    PROCESS(NPY_ULONGLONG, npy_ulonglong);
-    PROCESS(NPY_FLOAT, npy_float);
-    PROCESS(NPY_DOUBLE, npy_double);
-    PROCESS(NPY_LONGDOUBLE, npy_longdouble);
-    PROCESS(NPY_CFLOAT, npy_cfloat_wrapper);
-    PROCESS(NPY_CDOUBLE, npy_cdouble_wrapper);
-    PROCESS(NPY_CLONGDOUBLE, npy_clongdouble_wrapper);
+    SPTOOLS_FOR_EACH_DATA_TYPE_CODE(PROCESS)
 
 #undef PROCESS
 }
@@ -548,23 +509,7 @@ static PyObject *array_from_std_vector_and_free(int typenum, void *p)
         return obj;                                             \
     }
 
-    PROCESS(NPY_BOOL, npy_bool_wrapper);
-    PROCESS(NPY_BYTE, npy_byte);
-    PROCESS(NPY_UBYTE, npy_ubyte);
-    PROCESS(NPY_SHORT, npy_short);
-    PROCESS(NPY_USHORT, npy_ushort);
-    PROCESS(NPY_INT, npy_int);
-    PROCESS(NPY_UINT, npy_uint);
-    PROCESS(NPY_LONG, npy_long);
-    PROCESS(NPY_ULONG, npy_ulong);
-    PROCESS(NPY_LONGLONG, npy_longlong);
-    PROCESS(NPY_ULONGLONG, npy_ulonglong);
-    PROCESS(NPY_FLOAT, npy_float);
-    PROCESS(NPY_DOUBLE, npy_double);
-    PROCESS(NPY_LONGDOUBLE, npy_longdouble);
-    PROCESS(NPY_CFLOAT, npy_cfloat_wrapper);
-    PROCESS(NPY_CDOUBLE, npy_cdouble_wrapper);
-    PROCESS(NPY_CLONGDOUBLE, npy_clongdouble_wrapper);
+    SPTOOLS_FOR_EACH_DATA_TYPE_CODE(PROCESS)
 
 #undef PROCESS
 
@@ -607,7 +552,6 @@ extern "C" {
 
 #include "sparsetools_impl.h"
 
-#if PY_VERSION_HEX >= 0x03000000
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "_sparsetools",
@@ -627,15 +571,5 @@ PyObject *PyInit__sparsetools(void)
     import_array();
     return m;
 }
-#else
-PyMODINIT_FUNC init_sparsetools(void) {
-    PyObject *m;
-    m = Py_InitModule("_sparsetools", sparsetools_methods);
-    import_array();
-    if (m == NULL) {
-        Py_FatalError("can't initialize module _sparsetools");
-    }
-}
-#endif
 
 } /* extern "C" */
