@@ -31,6 +31,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+#include "_c99compat.h"
 
 
 static NPY_INLINE void
@@ -95,6 +96,22 @@ chebyshev_distance_double(const double *u, const double *v, const npy_intp n)
     return maxv;
 }
 
+static NPY_INLINE double
+weighted_chebyshev_distance_double(const double *u, const double *v,
+                                   const npy_intp n, const double *w)
+{
+    npy_intp i;
+    double maxv = 0.0;
+    for (i = 0; i < n; ++i) {
+        if (w[i] == 0.0) continue;
+        const double d = fabs(u[i] - v[i]);
+        if (d > maxv) {
+            maxv = d;
+        }
+    }
+    return maxv;
+}
+
 static NPY_INLINE double
 canberra_distance_double(const double *u, const double *v, const npy_intp n)
 {
@@ -397,8 +414,8 @@ weighted_minkowski_distance(const double *u, const double *v, const npy_intp n,
     npy_intp i = 0;
     double s = 0.0;
     for (i = 0; i < n; ++i) {
-        const double d = fabs(u[i] - v[i]) * w[i];
-        s += pow(d, p);
+        const double d = fabs(u[i] - v[i]);
+        s += pow(d, p) * w[i];
     }
     return pow(s, 1.0 / p);
 }
@@ -425,6 +442,78 @@ compute_mean_vector(double *res, const double *X, npy_intp num_rows, const npy_i
 }
 #endif
 
+#define DEFINE_CDIST(name, type) \
+    static int cdist_ ## name ## _ ## type(const type *XA, const type *XB, \
+                                           double *dm,                     \
+                                           const npy_intp num_rowsA,       \
+                                           const npy_intp num_rowsB,       \
+                                           const npy_intp num_cols)        \
+    {                                                                      \
+        Py_ssize_t i, j;                                                   \
+        for (i = 0; i < num_rowsA; ++i) {                                  \
+            const type *u = XA + num_cols * i;                             \
+            for (j = 0; j < num_rowsB; ++j, ++dm) {                        \
+                const type *v = XB + num_cols * j;                         \
+                *dm = name ## _distance_ ## type(u, v, num_cols);          \
+            }                                                              \
+        }                                                                  \
+        return 0;\
+    }
+
+DEFINE_CDIST(bray_curtis, double)
+DEFINE_CDIST(canberra, double)
+DEFINE_CDIST(chebyshev, double)
+DEFINE_CDIST(city_block, double)
+DEFINE_CDIST(euclidean, double)
+DEFINE_CDIST(jaccard, double)
+DEFINE_CDIST(jensenshannon, double)
+DEFINE_CDIST(sqeuclidean, double)
+
+DEFINE_CDIST(dice, char)
+DEFINE_CDIST(jaccard, char)
+DEFINE_CDIST(kulsinski, char)
+DEFINE_CDIST(rogerstanimoto, char)
+DEFINE_CDIST(russellrao, char)
+DEFINE_CDIST(sokalmichener, char)
+DEFINE_CDIST(sokalsneath, char)
+DEFINE_CDIST(yule, char)
+
+
+#define DEFINE_PDIST(name, type) \
+    static int pdist_ ## name ## _ ## type(const type *X, double *dm,       \
+                                           const npy_intp num_rows,         \
+                                           const npy_intp num_cols)         \
+    {                                                                       \
+        Py_ssize_t i, j;                                                    \
+        double *it = dm;                                                    \
+        for (i = 0; i < num_rows; ++i) {                                    \
+            const type *u = X + num_cols * i;                               \
+            for (j = i + 1; j < num_rows; ++j, it++) {                      \
+                const type *v = X + num_cols * j;                           \
+                *it = name ## _distance_ ## type(u, v, num_cols);           \
+            }                                                               \
+        }                                                                   \
+        return 0; \
+    }
+
+DEFINE_PDIST(bray_curtis, double)
+DEFINE_PDIST(canberra, double)
+DEFINE_PDIST(chebyshev, double)
+DEFINE_PDIST(city_block, double)
+DEFINE_PDIST(euclidean, double)
+DEFINE_PDIST(jaccard, double)
+DEFINE_PDIST(jensenshannon, double)
+DEFINE_PDIST(sqeuclidean, double)
+
+DEFINE_PDIST(dice, char)
+DEFINE_PDIST(jaccard, char)
+DEFINE_PDIST(kulsinski, char)
+DEFINE_PDIST(rogerstanimoto, char)
+DEFINE_PDIST(russellrao, char)
+DEFINE_PDIST(sokalmichener, char)
+DEFINE_PDIST(sokalsneath, char)
+DEFINE_PDIST(yule, char)
+
 static NPY_INLINE int
 pdist_mahalanobis(const double *X, double *dm, const npy_intp num_rows,
                   const npy_intp num_cols, const double *covinv)
@@ -449,6 +538,22 @@ pdist_mahalanobis(const double *X, double *dm, const npy_intp num_rows,
     return 0;
 }
 
+static NPY_INLINE int
+pdist_weighted_chebyshev(const double *X, double *dm, npy_intp num_rows,
+                         const npy_intp num_cols, const double *w)
+{
+    npy_intp i, j;
+
+    for (i = 0; i < num_rows; ++i) {
+        const double *u = X + (num_cols * i);
+        for (j = i + 1; j < num_rows; ++j, ++dm) {
+            const double *v = X + (num_cols * j);
+            *dm = weighted_chebyshev_distance_double(u, v, num_cols, w);
+        }
+    }
+    return 0;
+}
+
 static NPY_INLINE int
 pdist_cosine(const double *X, double *dm, const npy_intp num_rows,
              const npy_intp num_cols)
@@ -499,6 +604,15 @@ pdist_minkowski(const double *X, double *dm, npy_intp num_rows,
                 const npy_intp num_cols, const double p)
 {
     npy_intp i, j;
+    if (p == 1.0) {
+        return pdist_city_block_double(X, dm, num_rows, num_cols);
+    }
+    if (p == 2.0) {
+        return pdist_euclidean_double(X, dm, num_rows, num_cols);
+    }
+    if (sc_isinf(p)) {
+        return pdist_chebyshev_double(X, dm, num_rows, num_cols);
+    }
 
     for (i = 0; i < num_rows; ++i) {
         const double *u = X + (num_cols * i);
@@ -510,12 +624,44 @@ pdist_minkowski(const double *X, double *dm, npy_intp num_rows,
     return 0;
 }
 
+/* Old weighting type which is inconsistent with other distance metrics.
+   Remove in SciPy 1.8 along with wminkowski. */
+static NPY_INLINE int
+pdist_old_weighted_minkowski(const double *X, double *dm, npy_intp num_rows,
+                             const npy_intp num_cols, const double p, const double *w)
+{
+    npy_intp i, j;
+
+    // Covert from old style weights to new weights
+    double * new_weights = malloc(num_cols * sizeof(double));
+    if (!new_weights) {
+        return 1;
+    }
+    for (i = 0; i < num_cols; ++i) {
+        new_weights[i] = pow(w[i], p);
+    }
+
+    for (i = 0; i < num_rows; ++i) {
+        const double *u = X + (num_cols * i);
+        for (j = i + 1; j < num_rows; ++j, ++dm) {
+            const double *v = X + (num_cols * j);
+            *dm = weighted_minkowski_distance(u, v, num_cols, p, new_weights);
+        }
+    }
+    free(new_weights);
+    return 0;
+}
+
 static NPY_INLINE int
 pdist_weighted_minkowski(const double *X, double *dm, npy_intp num_rows,
                          const npy_intp num_cols, const double p, const double *w)
 {
     npy_intp i, j;
 
+    if (sc_isinf(p)) {
+        return pdist_weighted_chebyshev(X, dm, num_rows, num_cols, w);
+    }
+
     for (i = 0; i < num_rows; ++i) {
         const double *u = X + (num_cols * i);
         for (j = i + 1; j < num_rows; ++j, ++dm) {
@@ -610,6 +756,23 @@ dist_to_vector_from_squareform(const char *M, char *v, const npy_intp n, npy_int
     }
 }
 
+static NPY_INLINE int
+cdist_weighted_chebyshev(const double *XA, const double *XB, double *dm,
+                         const npy_intp num_rowsA, const npy_intp num_rowsB,
+                         const npy_intp num_cols, const double *w)
+{
+    npy_intp i, j;
+
+    for (i = 0; i < num_rowsA; ++i) {
+        const double *u = XA + (num_cols * i);
+        for (j = 0; j < num_rowsB; ++j, ++dm) {
+            const double *v = XB + (num_cols * j);
+            *dm = weighted_chebyshev_distance_double(u, v, num_cols, w);
+        }
+    }
+    return 0;
+}
+
 
 /** cdist */
 static NPY_INLINE int
@@ -693,6 +856,15 @@ cdist_minkowski(const double *XA, const double *XB, double *dm,
                 const npy_intp num_cols, const double p)
 {
     npy_intp i, j;
+    if (p == 1.0) {
+        return cdist_city_block_double(XA, XB, dm, num_rowsA, num_rowsB, num_cols);
+    }
+    if (p == 2.0) {
+        return cdist_euclidean_double(XA, XB, dm, num_rowsA, num_rowsB, num_cols);
+    }
+    if (sc_isinf(p)) {
+        return cdist_chebyshev_double(XA, XB, dm, num_rowsA, num_rowsB, num_cols);
+    }
 
     for (i = 0; i < num_rowsA; ++i) {
         const double *u = XA + (num_cols * i);
@@ -704,6 +876,37 @@ cdist_minkowski(const double *XA, const double *XB, double *dm,
     return 0;
 }
 
+/* Old weighting type which is inconsistent with other distance metrics.
+   Remove in SciPy 1.8 along with wminkowski. */
+static NPY_INLINE int
+cdist_old_weighted_minkowski(const double *XA, const double *XB, double *dm,
+                             const npy_intp num_rowsA, const npy_intp num_rowsB,
+                             const npy_intp num_cols, const double p,
+                             const double *w)
+{
+    npy_intp i, j;
+
+    // Covert from old style weights to new weights
+    double * new_weights = malloc(num_cols * sizeof(double));
+    if (!new_weights) {
+      return 1;
+    }
+
+    for (i = 0; i < num_cols; ++i) {
+      new_weights[i] = pow(w[i], p);
+    }
+
+    for (i = 0; i < num_rowsA; ++i) {
+        const double *u = XA + (num_cols * i);
+        for (j = 0; j < num_rowsB; ++j, ++dm) {
+            const double *v = XB + (num_cols * j);
+            *dm = weighted_minkowski_distance(u, v, num_cols, p, new_weights);
+        }
+    }
+    free(new_weights);
+    return 0;
+}
+
 static NPY_INLINE int
 cdist_weighted_minkowski(const double *XA, const double *XB, double *dm,
                          const npy_intp num_rowsA, const npy_intp num_rowsB,
@@ -712,6 +915,10 @@ cdist_weighted_minkowski(const double *XA, const double *XB, double *dm,
 {
     npy_intp i, j;
 
+    if (sc_isinf(p)) {
+        return cdist_weighted_chebyshev(XA, XB, dm, num_rowsA, num_rowsB, num_cols, w);
+    }
+
     for (i = 0; i < num_rowsA; ++i) {
         const double *u = XA + (num_cols * i);
         for (j = 0; j < num_rowsB; ++j, ++dm) {
@@ -757,75 +964,3 @@ cdist_hamming_char(const char *XA, const char *XB, double *dm,
     }
     return 0;
 }
-
-#define DEFINE_CDIST(name, type) \
-    static int cdist_ ## name ## _ ## type(const type *XA, const type *XB, \
-                                           double *dm,                     \
-                                           const npy_intp num_rowsA,       \
-                                           const npy_intp num_rowsB,       \
-                                           const npy_intp num_cols)        \
-    {                                                                      \
-        Py_ssize_t i, j;                                                   \
-        for (i = 0; i < num_rowsA; ++i) {                                  \
-            const type *u = XA + num_cols * i;                             \
-            for (j = 0; j < num_rowsB; ++j, ++dm) {                        \
-                const type *v = XB + num_cols * j;                         \
-                *dm = name ## _distance_ ## type(u, v, num_cols);          \
-            }                                                              \
-        }                                                                  \
-        return 0;\
-    }
-
-DEFINE_CDIST(bray_curtis, double)
-DEFINE_CDIST(canberra, double)
-DEFINE_CDIST(chebyshev, double)
-DEFINE_CDIST(city_block, double)
-DEFINE_CDIST(euclidean, double)
-DEFINE_CDIST(jaccard, double)
-DEFINE_CDIST(jensenshannon, double)
-DEFINE_CDIST(sqeuclidean, double)
-
-DEFINE_CDIST(dice, char)
-DEFINE_CDIST(jaccard, char)
-DEFINE_CDIST(kulsinski, char)
-DEFINE_CDIST(rogerstanimoto, char)
-DEFINE_CDIST(russellrao, char)
-DEFINE_CDIST(sokalmichener, char)
-DEFINE_CDIST(sokalsneath, char)
-DEFINE_CDIST(yule, char)
-
-
-#define DEFINE_PDIST(name, type) \
-    static int pdist_ ## name ## _ ## type(const type *X, double *dm,       \
-                                           const npy_intp num_rows,         \
-                                           const npy_intp num_cols)         \
-    {                                                                       \
-        Py_ssize_t i, j;                                                    \
-        double *it = dm;                                                    \
-        for (i = 0; i < num_rows; ++i) {                                    \
-            const type *u = X + num_cols * i;                               \
-            for (j = i + 1; j < num_rows; ++j, it++) {                      \
-                const type *v = X + num_cols * j;                           \
-                *it = name ## _distance_ ## type(u, v, num_cols);           \
-            }                                                               \
-        }                                                                   \
-        return 0; \
-    }
-
-DEFINE_PDIST(bray_curtis, double)
-DEFINE_PDIST(canberra, double)
-DEFINE_PDIST(chebyshev, double)
-DEFINE_PDIST(city_block, double)
-DEFINE_PDIST(euclidean, double)
-DEFINE_PDIST(jaccard, double)
-DEFINE_PDIST(jensenshannon, double)
-DEFINE_PDIST(sqeuclidean, double)
-
-DEFINE_PDIST(dice, char)
-DEFINE_PDIST(jaccard, char)
-DEFINE_PDIST(kulsinski, char)
-DEFINE_PDIST(rogerstanimoto, char)
-DEFINE_PDIST(russellrao, char)
-DEFINE_PDIST(sokalmichener, char)
-DEFINE_PDIST(sokalsneath, char)
-DEFINE_PDIST(yule, char)
